@@ -179,11 +179,12 @@ rebuild_child (GtkWidget  *self,
 
   button_child = NULL;
 
+  if (title && (!tooltip || !tooltip[0]))
+    tooltip = title;
+
   if (icon != NULL)
     {
       button_child = gtk_image_new_from_gicon (icon);
-      if (title != NULL)
-        gtk_widget_set_tooltip_text (GTK_WIDGET (self), title);
 
       gtk_widget_remove_css_class (self, "text-button");
       gtk_widget_add_css_class (self, "image-button");
@@ -192,8 +193,6 @@ rebuild_child (GtkWidget  *self,
     {
       button_child = gtk_label_new (title);
 
-      gtk_widget_set_tooltip_text (GTK_WIDGET (self), NULL);
-
       gtk_widget_remove_css_class (self, "image-button");
       gtk_widget_add_css_class (self, "text-button");
     }
@@ -356,8 +355,7 @@ panel_frame_switcher_drag_prepare_cb (PanelFrameSwitcher *self,
                                       double              y,
                                       GtkDragSource      *source)
 {
-  GtkStackPage *page = NULL;
-  GListModel *pages;
+  PanelWidget *page;
   GtkWidget *child;
   guint i = 0;
 
@@ -384,18 +382,17 @@ panel_frame_switcher_drag_prepare_cb (PanelFrameSwitcher *self,
         i++;
     }
 
-  pages = G_LIST_MODEL (panel_frame_get_pages (self->frame));
-  page = g_list_model_get_item (pages, i);
-  child = gtk_stack_page_get_child (page);
-  g_clear_object (&page);
+  page = panel_frame_get_page (self->frame, i);
 
-  if (!PANEL_IS_WIDGET (child) ||
-      !panel_widget_get_reorderable (PANEL_WIDGET (child)))
+  g_assert (!page || PANEL_IS_WIDGET (page));
+
+  if (!PANEL_IS_WIDGET (page) ||
+      !panel_widget_get_reorderable (PANEL_WIDGET (page)))
     return NULL;
 
-  self->drag_panel = PANEL_WIDGET (child);
+  self->drag_panel = page;
 
-  return gdk_content_provider_new_typed (PANEL_TYPE_WIDGET, child);
+  return gdk_content_provider_new_typed (PANEL_TYPE_WIDGET, page);
 }
 
 #define MAX_WIDTH  250.0
