@@ -51,6 +51,7 @@ typedef struct
   PanelDropControls *drop_controls;
 
   guint              closeable : 1;
+  guint              empty : 1;
 } PanelFramePrivate;
 
 #define SIZE_AT_END 50
@@ -73,6 +74,14 @@ enum {
   PROP_ORIENTATION,
 };
 
+enum {
+  ADOPT_WIDGET,
+  PAGE_CLOSED,
+  N_SIGNALS
+};
+
+
+static guint signals [N_SIGNALS];
 static GParamSpec *properties [N_PROPS];
 static GtkBuildableIface *parent_buildable;
 
@@ -125,7 +134,10 @@ panel_frame_close_page_cb (PanelFrame *self,
     adw_tab_view_set_selected_page (tab_view, tab_page);
 
   if (!_panel_widget_can_save (widget))
-    return FALSE;
+    {
+      g_signal_emit (self, signals [PAGE_CLOSED], 0, widget);
+      return FALSE;
+    }
 
   root = gtk_widget_get_root (GTK_WIDGET (self));
   delegate = panel_widget_get_save_delegate (widget);
@@ -356,6 +368,7 @@ panel_frame_update_actions (PanelFrame *self)
   PanelActionMuxer *action_group = NULL;
   PanelWidget *visible_child;
   GtkWidget *grid;
+  gboolean pinned;
 
   g_assert (PANEL_IS_FRAME (self));
 
@@ -363,15 +376,21 @@ panel_frame_update_actions (PanelFrame *self)
   visible_child = panel_frame_get_visible_child (self);
 
   if (visible_child != NULL)
-    action_group = _panel_widget_get_action_muxer (visible_child);
+    {
+      AdwTabPage *page = adw_tab_view_get_page (priv->tab_view, GTK_WIDGET (visible_child));
+
+      action_group = _panel_widget_get_action_muxer (visible_child);
+      pinned = adw_tab_page_get_pinned (page);
+    }
+
   gtk_widget_insert_action_group (GTK_WIDGET (self),
                                   "page",
                                   G_ACTION_GROUP (action_group));
 
-  gtk_widget_action_set_enabled (GTK_WIDGET (self), "page.move-right", grid  && visible_child);
-  gtk_widget_action_set_enabled (GTK_WIDGET (self), "page.move-left", grid && visible_child);
-  gtk_widget_action_set_enabled (GTK_WIDGET (self), "page.move-down", grid && visible_child);
-  gtk_widget_action_set_enabled (GTK_WIDGET (self), "page.move-up", grid && visible_child);
+  gtk_widget_action_set_enabled (GTK_WIDGET (self), "page.move-right", grid  && visible_child && !pinned);
+  gtk_widget_action_set_enabled (GTK_WIDGET (self), "page.move-left", grid && visible_child && !pinned);
+  gtk_widget_action_set_enabled (GTK_WIDGET (self), "page.move-down", grid && visible_child && !pinned);
+  gtk_widget_action_set_enabled (GTK_WIDGET (self), "page.move-up", grid && visible_child && !pinned);
   gtk_widget_action_set_enabled (GTK_WIDGET (self),
                                  "frame.close-page-or-frame",
                                  grid && (visible_child || priv->closeable));
@@ -555,6 +574,32 @@ panel_frame_update_drop (PanelFrame *self)
   panel_drop_controls_set_area (priv->drop_controls, area);
 }
 
+static gboolean
+panel_frame_real_adopt_widget (PanelFrame  *self,
+                               PanelWidget *widget)
+{
+  PanelDock *dock;
+
+  if ((dock = PANEL_DOCK (gtk_widget_get_ancestor (GTK_WIDGET (self), PANEL_TYPE_DOCK))))
+    {
+      if (!_panel_dock_can_adopt (dock, widget))
+        return GDK_EVENT_STOP;
+    }
+
+  return GDK_EVENT_PROPAGATE;
+}
+
+static gboolean
+panel_frame_can_adopt (PanelFrame  *self,
+                       PanelWidget *widget)
+{
+  gboolean ret = GDK_EVENT_PROPAGATE;
+
+  g_signal_emit (self, signals [ADOPT_WIDGET], 0, widget, &ret);
+
+  return ret == GDK_EVENT_PROPAGATE;
+}
+
 static void
 on_panel_drag_begin_cb (PanelFrame  *self,
                         PanelWidget *widget,
@@ -566,7 +611,8 @@ on_panel_drag_begin_cb (PanelFrame  *self,
   g_assert (PANEL_IS_WIDGET (widget));
   g_assert (PANEL_IS_DOCK (dock));
 
-  if (priv->header != NULL &&
+  if (panel_frame_can_adopt (self, widget) &&
+      priv->header != NULL &&
       panel_frame_header_can_drop (priv->header, widget))
     gtk_widget_show (GTK_WIDGET (priv->drop_controls));
 }
@@ -667,6 +713,35 @@ panel_frame_setup_menu_cb (AdwTabView *tab_view,
     panel_joined_menu_prepend_menu (joined, menu_model);
 }
 
+static void
+on_notify_selected_page_cb (PanelFrame *self,
+                            GParamSpec *pspec,
+                            AdwTabView *tab_view)
+{
+  PanelFramePrivate *priv = panel_frame_get_instance_private (self);
+  gboolean empty;
+
+  g_assert (PANEL_IS_FRAME (self));
+  g_assert (ADW_IS_TAB_VIEW (tab_view));
+
+  empty = adw_tab_view_get_selected_page (tab_view) == NULL;
+
+  if (empty != priv->empty)
+    {
+      priv->empty = empty;
+      g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_EMPTY]);
+    }
+}
+
+static void
+panel_frame_real_page_closed (PanelFrame  *self,
+                              PanelWidget *page)
+{
+  g_assert (PANEL_IS_FRAME (self));
+  g_assert (PANEL_IS_WIDGET (page));
+
+}
+
 static void
 panel_frame_compute_expand (GtkWidget *widget,
                             gboolean  *hexpand,
@@ -808,6 +883,9 @@ panel_frame_class_init (PanelFrameClass *klass)
   widget_class->unroot = panel_frame_unroot;
   widget_class->compute_expand = panel_frame_compute_expand;
 
+  klass->page_closed = panel_frame_real_page_closed;
+  klass->adopt_widget = panel_frame_real_adopt_widget;
+
   properties [PROP_CLOSEABLE] =
     g_param_spec_boolean ("closeable",
                           "Closeable",
@@ -840,6 +918,51 @@ panel_frame_class_init (PanelFrameClass *klass)
 
   g_object_class_override_property (object_class, PROP_ORIENTATION, "orientation");
 
+  /**
+   * PanelFrame::adopt-widget
+   * @self: a #PanelFrame
+   * @widget: a #PanelWidget
+   *
+   * This signal is emitted when the frame should decide if it can adopt
+   * a #PanelWidget dropped on the frame.
+   *
+   * If @GDK_EVENT_STOP is returned, then the widget will not be adopted.
+   *
+   * Returns: %GDK_EVENT_STOP or %GDK_EVENT_PROPAGATE
+   *
+   * Since: 1.2
+   */
+  signals [ADOPT_WIDGET] =
+    g_signal_new ("adopt-widget",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET (PanelFrameClass, adopt_widget),
+                  g_signal_accumulator_true_handled, NULL,
+                  NULL,
+                  G_TYPE_BOOLEAN,
+                  1,
+                  PANEL_TYPE_WIDGET);
+
+  /**
+   * PanelFrame::page-closed:
+   * @self: a #PanelFrame
+   * @widget: a #PanelWidget
+   *
+   * This signal is emitted when the page widget will be closed.
+   *
+   * Since: 1.2
+   */
+  signals [PAGE_CLOSED] =
+    g_signal_new ("page-closed",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET (PanelFrameClass, page_closed),
+                  NULL, NULL,
+                  NULL,
+                  G_TYPE_NONE,
+                  1,
+                  PANEL_TYPE_WIDGET);
+
   gtk_widget_class_set_css_name (widget_class, "panelframe");
   gtk_widget_class_set_layout_manager_type (widget_class, GTK_TYPE_BIN_LAYOUT);
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/libpanel/panel-frame.ui");
@@ -878,11 +1001,19 @@ panel_frame_init (PanelFrame *self)
   PanelFramePrivate *priv = panel_frame_get_instance_private (self);
   PanelJoinedMenu *menu;
 
+  priv->empty = TRUE;
+
   gtk_widget_init_template (GTK_WIDGET (self));
 
   _panel_dock_update_orientation (GTK_WIDGET (self),
                                   gtk_orientable_get_orientation (GTK_ORIENTABLE (self)));
 
+  g_signal_connect_object (priv->tab_view,
+                           "notify::selected-page",
+                           G_CALLBACK (on_notify_selected_page_cb),
+                           self,
+                           G_CONNECT_SWAPPED);
+
   /* Locate GtkStack within tab view and alter homogeneous
    * values so that we have more flexibility in sizing.
    */
@@ -936,7 +1067,6 @@ panel_frame_add_before (PanelFrame  *self,
   GtkWidget *dock = NULL;
   GtkWidget *dock_child;
   GtkWidget *grid;
-  gboolean empty;
   int position;
 
   g_return_if_fail (PANEL_IS_FRAME (self));
@@ -957,7 +1087,6 @@ panel_frame_add_before (PanelFrame  *self,
       position = adw_tab_view_get_n_pages (priv->tab_view);
     }
 
-  empty = panel_frame_get_empty (self);
   page = adw_tab_view_insert (priv->tab_view, GTK_WIDGET (panel), position);
 
   g_object_bind_property (panel, "title", page, "title", G_BINDING_SYNC_CREATE);
@@ -975,9 +1104,6 @@ panel_frame_add_before (PanelFrame  *self,
 
   panel_frame_update_actions (self);
 
-  if (empty)
-    g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_EMPTY]);
-
   if (dock_child != NULL && dock != NULL)
     {
       PanelArea dockpos = panel_dock_child_get_area (PANEL_DOCK_CHILD (dock_child));
@@ -1076,7 +1202,7 @@ panel_frame_get_empty (PanelFrame *self)
 
   g_return_val_if_fail (PANEL_IS_FRAME (self), FALSE);
 
-  return adw_tab_view_get_n_pages (priv->tab_view) == 0;
+  return priv->empty;
 }
 
 /**
@@ -1121,6 +1247,31 @@ panel_frame_set_visible_child (PanelFrame  *self,
     adw_tab_view_set_selected_page (priv->tab_view, page);
 }
 
+/**
+ * panel_frame_set_child_pinned:
+ * @self: a #PanelFrame
+ * @child: (not nullable): a #PanelWidget
+ * @pinned: if @widget should be pinned
+ *
+ * Set pinned state of @child.
+ *
+ * Since: 1.2
+ */
+void
+panel_frame_set_child_pinned (PanelFrame  *self,
+                              PanelWidget *child,
+                              gboolean     pinned)
+{
+  PanelFramePrivate *priv = panel_frame_get_instance_private (self);
+  AdwTabPage *page;
+
+  g_return_if_fail (PANEL_IS_FRAME (self));
+  g_return_if_fail (PANEL_IS_WIDGET (child));
+
+  if ((page = adw_tab_view_get_page (priv->tab_view, GTK_WIDGET (child))))
+    adw_tab_view_set_page_pinned (priv->tab_view, page, pinned);
+}
+
 AdwTabView *
 _panel_frame_get_tab_view (PanelFrame *self)
 {
